/**
 * @cond doxygenLibsbmlInternal
 *
 * @file    StrictUnitConsistencyConstraintsDeclared.cxx
 * @brief   Declarations of constraints
 * @author  SBML Team
 * 
 * <!--------------------------------------------------------------------------
 * This file is part of libSBML.  Please visit http://sbml.org for more
 * information about SBML, and the latest version of libSBML.
 *
 * Copyright (C) 2019 jointly by the following organizations:
 *     1. California Institute of Technology, Pasadena, CA, USA
 *     2. University of Heidelberg, Heidelberg, Germany
 *
 * Copyright (C) 2013-2018 jointly by the following organizations:
 *     1. California Institute of Technology, Pasadena, CA, USA
 *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
 *     3. University of Heidelberg, Heidelberg, Germany
 *
 * Copyright (C) 2009-2013 jointly by the following organizations: 
 *     1. California Institute of Technology, Pasadena, CA, USA
 *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
 *  
 * Copyright (C) 2006-2008 by the California Institute of Technology,
 *     Pasadena, CA, USA 
 *  
 * Copyright (C) 2002-2005 jointly by the following organizations:
 *     1. California Institute of Technology, Pasadena, CA, USA
 *     2. Japan Science and Technology Agency, Japan
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.  A copy of the license agreement is provided
 * in the file named "LICENSE.txt" included with this software distribution
 * and also available online as http://sbml.org/software/libsbml/license.html
 * ---------------------------------------------------------------------- -->*/

addConstraint(new VConstraintModel9920217(*this));

addConstraint(new VConstraintModel9920218(*this));

addConstraint(new VConstraintModel9920219(*this));

addConstraint(new VConstraintModel9920220(*this));

addConstraint(new VConstraintModel9920221(*this));

addConstraint(new VConstraintCompartment9920509(*this));

addConstraint(new VConstraintSpecies9920608(*this));

addConstraint(new VConstraintParameter9920701(*this));

addConstraint(new VConstraintModel9999130(*this));

addConstraint(new VConstraintAssignmentRule9999505(*this));

addConstraint(new VConstraintRateRule9999505(*this));

addConstraint(new VConstraintInitialAssignment9999505(*this));

addConstraint(new VConstraintKineticLaw9999505(*this));

addConstraint(new VConstraintEvent9999505(*this));

addConstraint(new VConstraintPriority9999505(*this));

addConstraint(new VConstraintEventAssignment9999505(*this));

addConstraint(new VConstraintCompartment9999508(*this));

addConstraint(new VConstraintSpecies9999508(*this));

addConstraint(new VConstraintParameter9999508(*this));

addConstraint(new VConstraintModel9999506(*this));

addConstraint(new VConstraintModel9999507(*this));

addConstraint(new ArgumentsUnitsCheck(9910501, *this));

addConstraint(new ArgumentsUnitsCheckWarnings(9999502, *this));

addConstraint(new PowerUnitsCheck(9999503, *this));

addConstraint(new ExponentUnitsCheck(9999504, *this));

addConstraint(new KineticLawUnitsCheck(9910503, *this));

addConstraint(new VConstraintAssignmentRule9910511(*this));

addConstraint(new VConstraintAssignmentRule9910512(*this));

addConstraint(new VConstraintAssignmentRule9910513(*this));

addConstraint(new VConstraintAssignmentRule9910514(*this));

addConstraint(new VConstraintInitialAssignment9910521(*this));

addConstraint(new VConstraintInitialAssignment9910522(*this));

addConstraint(new VConstraintInitialAssignment9910523(*this));

addConstraint(new VConstraintInitialAssignment9910524(*this));

addConstraint(new VConstraintRateRule9910531(*this));

addConstraint(new VConstraintRateRule9910532(*this));

addConstraint(new VConstraintRateRule9910533(*this));

addConstraint(new VConstraintRateRule9910534(*this));

addConstraint(new VConstraintKineticLaw9910541(*this));

addConstraint(new VConstraintSpecies9910542(*this));

addConstraint(new VConstraintEvent9910551(*this));

addConstraint(new VConstraintEventAssignment9910561(*this));

addConstraint(new VConstraintEventAssignment9910562(*this));

addConstraint(new VConstraintEventAssignment9910563(*this));

addConstraint(new VConstraintEventAssignment9910564(*this));

addConstraint(new VConstraintPriority9910565(*this));

addConstraint(new VConstraintCompartment9920511(*this));

addConstraint(new VConstraintCompartment9920512(*this));

addConstraint(new VConstraintCompartment9920513(*this));

addConstraint(new VConstraintCompartment9920518(*this));

addConstraint(new VConstraintSpecies9920616(*this));

addConstraint(new VConstraintParameter9920702(*this));

addConstraint(new VConstraintLocalParameter9920702(*this));
/** @endcond */

