/**
 * @cond doxygenLibsbmlInternal
 *
 * @file    UnitConsistencyConstraintsDeclared.cxx
 * @brief   Declarations of constraints
 * @author  SBML Team
 * 
 * <!--------------------------------------------------------------------------
 * This file is part of libSBML.  Please visit http://sbml.org for more
 * information about SBML, and the latest version of libSBML.
 *
 * Copyright (C) 2019 jointly by the following organizations:
 *     1. California Institute of Technology, Pasadena, CA, USA
 *     2. University of Heidelberg, Heidelberg, Germany
 *
 * Copyright (C) 2013-2018 jointly by the following organizations:
 *     1. California Institute of Technology, Pasadena, CA, USA
 *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
 *     3. University of Heidelberg, Heidelberg, Germany
 *
 * Copyright (C) 2009-2013 jointly by the following organizations: 
 *     1. California Institute of Technology, Pasadena, CA, USA
 *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
 *  
 * Copyright (C) 2006-2008 by the California Institute of Technology,
 *     Pasadena, CA, USA 
 *  
 * Copyright (C) 2002-2005 jointly by the following organizations:
 *     1. California Institute of Technology, Pasadena, CA, USA
 *     2. Japan Science and Technology Agency, Japan
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.  A copy of the license agreement is provided
 * in the file named "LICENSE.txt" included with this software distribution
 * and also available online as http://sbml.org/software/libsbml/license.html
 * ---------------------------------------------------------------------- -->*/

addConstraint(new VConstraintModel20217(*this));

addConstraint(new VConstraintModel20218(*this));

addConstraint(new VConstraintModel20219(*this));

addConstraint(new VConstraintModel20220(*this));

addConstraint(new VConstraintModel20221(*this));

addConstraint(new VConstraintModel20233(*this));

addConstraint(new VConstraintCompartment20507(*this));

addConstraint(new VConstraintCompartment20508(*this));

addConstraint(new VConstraintCompartment20509(*this));

addConstraint(new VConstraintSpecies20608(*this));

addConstraint(new VConstraintParameter20701(*this));

addConstraint(new VConstraintModel99130(*this));

addConstraint(new VConstraintAssignmentRule99505(*this));

addConstraint(new VConstraintRateRule99505(*this));

addConstraint(new VConstraintInitialAssignment99505(*this));

addConstraint(new VConstraintKineticLaw99505(*this));

addConstraint(new VConstraintEvent99505(*this));

addConstraint(new VConstraintPriority99505(*this));

addConstraint(new VConstraintTrigger99505(*this));

addConstraint(new VConstraintEventAssignment99505(*this));

addConstraint(new VConstraintStoichiometryMath99505(*this));

addConstraint(new VConstraintConstraint99505(*this));

addConstraint(new VConstraintCompartment99508(*this));

addConstraint(new VConstraintSpecies99508(*this));

addConstraint(new VConstraintParameter99508(*this));

addConstraint(new VConstraintModel99506(*this));

addConstraint(new VConstraintModel99507(*this));

addConstraint(new VConstraintInitialAssignment99509(*this));

addConstraint(new VConstraintAlgebraicRule99509(*this));

addConstraint(new VConstraintAssignmentRule99509(*this));

addConstraint(new VConstraintRateRule99509(*this));

addConstraint(new VConstraintKineticLaw99509(*this));

addConstraint(new VConstraintConstraint99509(*this));

addConstraint(new VConstraintTrigger99509(*this));

addConstraint(new VConstraintPriority99509(*this));

addConstraint(new VConstraintDelay99509(*this));

addConstraint(new VConstraintFunctionDefinition99509(*this));

addConstraint(new VConstraintEventAssignment99509(*this));

addConstraint(new ArgumentsUnitsCheck(10501, *this));

addConstraint(new ArgumentsUnitsCheckWarnings(99502, *this));

addConstraint(new PowerUnitsCheck(99503, *this));

addConstraint(new ExponentUnitsCheck(99504, *this));

addConstraint(new KineticLawUnitsCheck(10503, *this));

addConstraint(new VConstraintAssignmentRule10511(*this));

addConstraint(new VConstraintAssignmentRule10512(*this));

addConstraint(new VConstraintAssignmentRule10513(*this));

addConstraint(new VConstraintAssignmentRule10514(*this));

addConstraint(new VConstraintInitialAssignment10521(*this));

addConstraint(new VConstraintInitialAssignment10522(*this));

addConstraint(new VConstraintInitialAssignment10523(*this));

addConstraint(new VConstraintInitialAssignment10524(*this));

addConstraint(new VConstraintRateRule10531(*this));

addConstraint(new VConstraintRateRule10532(*this));

addConstraint(new VConstraintRateRule10533(*this));

addConstraint(new VConstraintRateRule10534(*this));

addConstraint(new VConstraintKineticLaw10541(*this));

addConstraint(new VConstraintSpecies10542(*this));

addConstraint(new VConstraintEvent10551(*this));

addConstraint(new VConstraintEventAssignment10561(*this));

addConstraint(new VConstraintEventAssignment10562(*this));

addConstraint(new VConstraintEventAssignment10563(*this));

addConstraint(new VConstraintEventAssignment10564(*this));

addConstraint(new VConstraintPriority10565(*this));

addConstraint(new VConstraintCompartment20511(*this));

addConstraint(new VConstraintCompartment20512(*this));

addConstraint(new VConstraintCompartment20513(*this));

addConstraint(new VConstraintCompartment20518(*this));

addConstraint(new VConstraintSpecies20616(*this));

addConstraint(new VConstraintParameter20702(*this));

addConstraint(new VConstraintKineticLaw99127(*this));

addConstraint(new VConstraintKineticLaw99128(*this));


/** @endcond */

