// This file is autogenerated and will be overwritten by the configuration process

#include "local-downcast-namespaces-arrays.cpp"
#include "local-downcast-namespaces-comp.cpp"
#include "local-downcast-namespaces-distrib.cpp"
#include "local-downcast-namespaces-fbc.cpp"
#include "local-downcast-namespaces-groups.cpp"
#include "local-downcast-namespaces-layout.cpp"
#include "local-downcast-namespaces-multi.cpp"
#include "local-downcast-namespaces-qual.cpp"
#include "local-downcast-namespaces-render.cpp"
#include "local-downcast-namespaces-req.cpp"
#include "local-downcast-namespaces-spatial.cpp"

