/**
 * @file    libsbml-javadoc-stylesheet.css
 * @brief   Stylesheet used by libSBML for Javadoc documentation
 * @author  Michael Hucka
 *
 * This file is part of libSBML.  Please visit http://sbml.org for more
 * information about SBML, and the latest version of libSBML.
 */

@import url('libsbml-reset-stylesheet.css');
@import url('libsbml-base-stylesheet.css');
@import url('libsbml-package-stylesheet.css');

body {
  overflow: visible;
}

pre
{
    margin-top: 1em;
    margin-bottom: 1em;
}

a
{
    font-weight: bold;
}

a code, 
code a 
{
    color: #1133a9;
    text-decoration: none;
    font-weight: bold;
}

table[summary=""] td
{
    padding: 1px;
}

table[summary=""]
{
    margin-top: 1em;
}

td.NavBarCell1 table[summary=""]
{
    margin-top: 1px;
}

td.NavBarCell1 a font.NavBarFont1
{
    font-size: 8pt;
    border: 1px solid #999;
    border-radius: 6px;
    -moz-border-radius: 6px;
    -webkit-border-radius: 6px;
    margin-right: 4px;
    padding-left: 4px;
    padding-right: 4px;
}

td.NavBarCell1 a[href*="package-summary"] font.NavBarFont1,
td.NavBarCell1 a[href*="extensions-summary"] font.NavBarFont1
{
    letter-spacing: -1px;
}

font.NavBarFont1Rev b,
a[href="overview-summary.html"] font.NavBarFont1.top-level,
td.NavBarCell1 a[href="extensions-summary.html"] font.NavBarFont1.selected,
td.NavBarCell1 a[href="../../../org/sbml/libsbml/package-summary.html"] font.NavBarFont1.selected
{
    color: white;
    background-color: #959595;
    border: 1px solid #999;
    border-radius: 6px;
    -moz-border-radius: 6px;
    -webkit-border-radius: 6px;
    box-shadow: 1px 1px 1px #444 inset;
    -moz-box-shadow: 1px 1px 1px #444 inset;
    -webkit-box-shadow: 1px 1px 1px #444 inset;;
}

font.NavBarFont1Rev b
{ 
    margin-right: 3px;
    padding-left: 3px;
    padding-right: 3px;
    text-decoration: none;
}

tr.TableRowColor td 
{
    color: #666;
    border: none;
    border-bottom: 1px solid #bbb;
    vertical-align: top;
    padding-top: 4px;
    line-height: 150%;
    padding-left: 4px;
    padding-right: 4px;
} 

tr.TableRowColor td code 
{
    color: #444;
} 

tr.TableRowColor a[title="class in org.sbml.libsbml"],
tr.TableRowColor a[href$="org.sbml.libsbml"],
tr.TableRowColor b a[href*="org/sbml/libsbml"]
{
    font-family: "Anonymous Pro", "Courier New", Courier, monospace, fixed;
    font-weight: bold;
    font-size: 10pt;
    line-height: 150%;
    vertical-align: top;
}

a[title="class in org.sbml.libsbml"],
a[title="interface in org.sbml.libsbml"]
{
    font-family: "Anonymous Pro", "Courier New", Courier, monospace, fixed;
    font-weight: bold;
    font-size: 10pt;
}

tr.TableSubHeadingColor th 
{
    border: none;
    border-top: 1px solid #bbb;
    background-color: #d0d0d0;
    padding: 5px;
}

tr.TableHeadingColor th 
{
    color: black;
    letter-spacing: 1px;
    font-family: "Noto Sans", Verdana, Helvetica, Arial, Sans, sans-serif;
    font-size: 18px;
    padding: 5px 2px;
    background-color: #ddd;
    line-height: 150%;
    border-bottom: 1px solid #aaa;
    border-top: 3px solid #888;
    border-left: none;
    border-right: none;
}

tr.TableHeadingColor th font[size="+2"]
{
    font-size: 18px;
}

tr.TableHeadingColor th a[href*="org/sbml/libsbml"],
tr.TableSubHeadingColor th a[href*="org/sbml/libsbml"]
{
    font-family: "Anonymous Pro", "Courier New", Courier, monospace, fixed;
    font-weight: bold;
}

tr.TableHeadingColor th font b 
{
    font-size: 18px;
}

tr.TableSubHeadingColor th 
{
    border: none;
    border-top: 1px solid #bbb;
    background-color: #d0d0d0;
    padding: 5px;
}

/* Javadoc method sections */

body dl dd {
    margin-left: 3em;
}


/* The next one is to tighten vertical spacing in Java parameter
lists in method descriptions */

dl dd dl dd p 
{
    padding: 2px;
    margin: 0px;
}

/* The next one is to tighten vertical spacing in Java parameter
lists in method descriptions */

dl dd dl dd p 
{
    padding: 2px 2px 0.5em 2px;
    margin: 0px;
}

dl dd a[href*="org/sbml/libsbml"]
{
    font-family: "Anonymous Pro", "Courier New", Courier, monospace, fixed;
    font-weight: bold;
}

/* This next one catches the most common use of li for us inside
method documentation, namely for return types.  Hacky, this is. */

dl dd dl dd ul li 
{
    margin-top: 0.25em;
    margin-bottom: 0.25em;
}

td, th
{
    padding: 2px;
}

td.footer 
{
    font-family: "Noto Sans", Verdana, Helvetica, Arial, Sans, sans-serif;
    font-size: 12px;
    padding-left: 5px;
    padding-right: 5px;
}

img.footer 
{
}

/* The following is based on the code our swigdoc.py emits for @image. */

center.image
{
    margin-top: 1.5em;
    margin-bottom: 1em;
}

.footercomment 
{
    font-family: "Noto Sans", Verdana, Helvetica, Arial, Sans, sans-serif;
    font-size: x-small;
    font-style: italic;
    color: #555555;
}

table[border="1"] 
{
    border: none;
    border-bottom: 1px solid #666;
}

td.NavBarCell1,
td.NavBarCell2,
td.NavBarCell3 
{
    background-color: #eee;
    margin: 0px;
}

td.NavBarCell3 
{  
    padding: 4px;
}

td.NavBarCell1Rev 
{
    background-color: #eee !important; 
}

td[align="right"] 
{
    background-color: #eee;  
    vertical-align: bottom;
    padding-top: 2px;
    padding-bottom: 2px;
}


/* The next two are for the list of classes in the left-hand frame. */

/* The body:not selector catches the left-hand list of all classes, 
by virtue of the hack that Javadoc doesn't give it an onload
attribute like it does for the other pages. */

body:not([onload])
{
    margin-left: 5px;
}

font.FrameItemFont a
{
    font-family: "Noto Sans", Verdana, Helvetica, Arial, Sans, sans-serif;
    font-weight: normal;
    font-size: 9pt;
    margin-left: 10px;
}

/* Hide the "libSBML 5.x.x in the navigation bar because it takes
up space and it's not really critical.
*/

td[rowspan="3"]
{
    display: none;
}

/* The next two items remove <hr> lines (because Javadoc loves to 
insert them all over the place) and adds a line after (only) top-level
<dl> elements.  The result is that documentation for functions in
Javadoc get lines between them, but the hardwired-by-Javadoc <hr>
elements are made invisible. */

body hr,
body > hr
{
    display: none;
}

body > dl
{
    padding-bottom: 1em;
    border-bottom: 1px solid #bbb;
}

.warning
{
    color: #9b0000;
}

/* This next one works in concert with some code in our sbml.js, to
fix a problem with Javadoc's formatting of multiple custom tags. */

p.note
{
    margin-top: 0.5em;
}

div.fragment
{
    border: none;
    padding: 0;
    background-color: white;
}

.deprecated
{
    color: #C35817;
}

img.big-icon {
    vertical-align: -40%;
    margin-right: 5px;
    border: none;
}

/* Changes introduced in Javadoc after version 1.6. */

.indexContainer ul {
    list-style: none;
    padding-top: 0px;
}

.indexContainer ul li {
    margin-left: 5px;
    margin: 0;
}

.indexContainer ul a {
    font-family: "Noto Sans", Verdana, Helvetica, Arial, Sans, sans-serif;
    font-weight: normal;
    font-size: 9pt;
    padding: 0;
    margin: 0;
    margin-left: 10px;
}
